// -*-c++-*-
/* $Id: classfsd.h,v 1.3 2001/09/20 06:21:25 dm Exp $ */

/*
 *
 * Copyright (C) 2001 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "nfsserv.h"
#include "crypt.h"
#include "aes.h"
#include "nfs3_prot.h"
#include "afsnode.h"
#include "qhash.h"
#include "nfstrans.h"

/*
 * afs.C
 */
class afsroot : public afsdir {
  sfs_aid lastaid;
  qhash<sfs_aid, ref<afsdir> > utab;
protected:
  afsroot () : afsdir (NULL) { mtime.seconds = mtime.nseconds = 0; }
public:
  static ref<afsroot> alloc () { return New refcounted<afsroot> (); }

  void mkfattr3 (fattr3 *, sfs_aid aid);
  void bumpmtime () { mtime.seconds++; };
  afsnode *lookup (const str &name, sfs_aid aid);
  void nfs_remove (svccb *sbp);

  bool entryok (afsdirentry *de, sfs_aid aid);
  afsdirentry *firstentry (sfs_aid aid);
  afsdirentry *nextentry (afsdirentry *de, sfs_aid aid);

  ptr<afslink> aidlink (sfs_aid aid, const str &contents, const str &name);
  bool aidunlink (sfs_aid aid, const str &name);
};

extern ptr<afsroot> rootdir;
extern ptr<afsdir> uiddir;
void afs_init ();
bool uid_mkdir (sfs_aid aid, str name);
void uid_rmdir (sfs_aid aid, str name);

/*
 * adm.C
 */
extern int ping_timeout;
bool killserv (sfs_aid, str name);
bool detach (sfs_aid, str name);
void adm_init ();

/*
 * classfsd.C
 */
extern bool opt_nodaemon;
extern str mntpath;
extern ptr<axprt_unix> nmx;
extern ptr<aclnt> nmc;

#define UDNAME ".user"
inline str
mountpoint (sfs_aid aid, str name)
{
  return mntpath << "/" UDNAME "/" << aid << "/" << name;
}
inline str
mountpoint (const authunix_parms *aup, str name)
{
  return mountpoint (aup2aid (aup), name);
}
